-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS Galaxy (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    name TEXT NOT NULL,
    seed INTEGER NOT NULL,
    main INTEGER NOT NULL
);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS Region (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    name TEXT NOT NULL,
    pos Vector2f NOT NULL,
    galaxyId INTEGER NOT NULL,

    FOREIGN KEY (galaxyId) REFERENCES Galaxy (id)
);

CREATE INDEX IF NOT EXISTS Region_galaxyId_idx ON Region(galaxyId);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS Faction (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    name TEXT NOT NULL,
    description TEXT NOT NULL,
    color REAL NOT NULL,

    UNIQUE(name)
);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS FactionPlacement (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    factionId INTEGER NOT NULL,
    nodeCountMin INTEGER NOT NULL,
    nodeCountMax INTEGER NOT NULL,
    nodeSizeMin INTEGER NOT NULL,
    nodeSizeMax INTEGER NOT NULL,
    placeHome BOOLEAN NOT NULL,

    FOREIGN KEY (factionId) REFERENCES Faction (id)
);

CREATE INDEX IF NOT EXISTS FactionPlacement_factionId_idx ON FactionPlacement(factionId);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS System (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    name TEXT NOT NULL,
    pos Vector2f NOT NULL,
    seed INTEGER NOT NULL,
    galaxyId INTEGER NOT NULL,
    regionId INTEGER NOT NULL,
    factionId INTEGER,

    FOREIGN KEY (galaxyId) REFERENCES Galaxy (id),
    FOREIGN KEY (regionId) REFERENCES Region (id),
    FOREIGN KEY (factionId) REFERENCES Faction (id)
);

CREATE INDEX IF NOT EXISTS System_galaxyId_idx ON System(galaxyId);
CREATE INDEX IF NOT EXISTS System_regionId_idx ON System(regionId);
CREATE INDEX IF NOT EXISTS System_factionId_idx ON System(factionId);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS FactionHome (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    systemId INTEGER NOT NULL,
    factionId INTEGER NOT NULL,

    FOREIGN KEY (systemId) REFERENCES System (id),
    FOREIGN KEY (factionId) REFERENCES Faction (id),

    UNIQUE(systemId, factionId)
);

CREATE INDEX IF NOT EXISTS FactionHome_systemId_idx ON FactionHome(systemId);
CREATE INDEX IF NOT EXISTS FactionHome_factionId_idx ON FactionHome(factionId);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS SystemConnection (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    galaxyId INTEGER NOT NULL,
    systemId INTEGER NOT NULL,
    otherId INTEGER NOT NULL,

    FOREIGN KEY (galaxyId) REFERENCES Galaxy (id),
    FOREIGN KEY (systemId) REFERENCES System (id),
    FOREIGN KEY (otherId) REFERENCES System (id)
);

CREATE INDEX IF NOT EXISTS SystemConnection_galaxyId_idx ON SystemConnection(galaxyId);
CREATE INDEX IF NOT EXISTS SystemConnection_systemId_idx ON SystemConnection(systemId);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS SystemPlayerExplored (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    playerId INTEGER NOT NULL,
    systemId INTEGER NOT NULL,
    exploredAt TEXT NOT NULL,

    FOREIGN KEY (playerId) REFERENCES Player (id),
    FOREIGN KEY (systemId) REFERENCES System (id),

    UNIQUE(playerId, systemId)
);

CREATE INDEX IF NOT EXISTS SystemPlayerExplored_playerId_idx ON SystemPlayerExplored(playerId);
CREATE INDEX IF NOT EXISTS SystemPlayerExplored_systemId_idx ON SystemPlayerExplored(systemId);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS PlayerHomeSystem (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    galaxyId INTEGER NOT NULL,
    systemId INTEGER NOT NULL,
    sectorId INTEGER NOT NULL,

    FOREIGN KEY (galaxyId) REFERENCES Galaxy (id),
    FOREIGN KEY (systemId) REFERENCES System (id),
    FOREIGN KEY (sectorId) REFERENCES Sector (id)
);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS PlayerLocation (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    galaxyId INTEGER NOT NULL,
    systemId INTEGER NOT NULL,
    sectorId INTEGER NOT NULL,

    FOREIGN KEY (galaxyId) REFERENCES Galaxy (id),
    FOREIGN KEY (systemId) REFERENCES System (id),
    FOREIGN KEY (sectorId) REFERENCES Sector (id)
);

-- ====================================================================================================================
CREATE VIEW IF NOT EXISTS SystemView (
    'id',
    'name',
    'pos',
    'galaxyId',
    'regionId',
    'factionId',
	'exploredAt',
	'playerId'
)
AS
    SELECT
        System.id as id,
        System.name as name,
        System.pos as pos,
        System.galaxyId as galaxyId,
        System.regionId as regionId,
        System.factionId as factionId,
        SystemPlayerExplored.exploredAt as exploredAt,
        SystemPlayerExplored.playerId as playerId
    FROM
        System
    LEFT JOIN
        SystemPlayerExplored
    ON
        SystemPlayerExplored.systemId = System.id;

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS Sector (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    systemId INTEGER NOT NULL,
    name TEXT NOT NULL,
    pos Vector2f NOT NULL,
    seed INTEGER NOT NULL,
    icon TEXT NOT NULL,
    visible BOOLEAN NOT NULL,

    FOREIGN KEY (systemId) REFERENCES System (id)
);

CREATE INDEX IF NOT EXISTS Sector_systemId_idx ON Sector(systemId);
CREATE INDEX IF NOT EXISTS Sector_visible_idx ON Sector(visible);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS SectorPlayerExplored (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    playerId INTEGER NOT NULL,
    sectorId INTEGER NOT NULL,
    exploredAt TEXT NOT NULL,

    FOREIGN KEY (playerId) REFERENCES Player (id),
    FOREIGN KEY (sectorId) REFERENCES System (id),

    UNIQUE(playerId, sectorId)
);

CREATE INDEX IF NOT EXISTS SectorPlayerExplored_playerId_idx ON SectorPlayerExplored(playerId);
CREATE INDEX IF NOT EXISTS SectorPlayerExplored_systemId_idx ON SectorPlayerExplored(sectorId);

-- ====================================================================================================================
CREATE VIEW IF NOT EXISTS SectorView (
    'id',
    'name',
    'pos',
    'seed',
    'icon',
    'visible',
    'systemId',
	'exploredAt',
	'playerId'
)
AS
    SELECT
        Sector.id as id,
        Sector.name as name,
        Sector.pos as pos,
        Sector.seed as seed,
        Sector.icon as icon,
        Sector.visible as visible,
        Sector.systemId as systemId,
        SectorPlayerExplored.exploredAt as exploredAt,
        SectorPlayerExplored.playerId as playerId
    FROM
        Sector
    LEFT JOIN
        SectorPlayerExplored
    ON
        SectorPlayerExplored.sectorId = Sector.id;

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS SystemGate (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    systemId INTEGER NOT NULL,
    sectorId INTEGER NOT NULL,
    targetSystemId INTEGER NOT NULL,
    targetSectorId INTEGER NOT NULL,

    FOREIGN KEY (systemId) REFERENCES System (id),
    FOREIGN KEY (sectorId) REFERENCES Sector (id),
    FOREIGN KEY (targetSystemId) REFERENCES System (id),
    FOREIGN KEY (targetSectorId) REFERENCES Sector (id)
);

CREATE INDEX IF NOT EXISTS SystemGate_systemId_idx ON SystemGate(systemId);
CREATE INDEX IF NOT EXISTS SystemGate_targetSystemId_idx ON SystemGate(targetSystemId);

-- ====================================================================================================================
CREATE TABLE IF NOT EXISTS SystemPlanet (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    systemId INTEGER NOT NULL,
    name TEXT NOT NULL,
    pos Vector2f NOT NULL,
    type TEXT NOT NULL,
    radius REAL NOT NULL,
    parent INTEGER,

    FOREIGN KEY (systemId) REFERENCES System (id),
    FOREIGN KEY (parent) REFERENCES SystemPlanet (id)
);
